////////////////////////////////////////////////////////////////////
// Asteroids clone.
//
// Written by Vladeta Stojanovic (0602920@live.abertay.ac.uk)
//
// Version: 181011_01
////////////////////////////////////////////////////////////////////

#include "ship.h"
#include "asteroid.h"
#include "bullet.h"
#include "astMaths.h"
#include "timer.h"
using namespace std;

/////////////////////////////////////
// Global game variables go here
/////////////////////////////////////
int gameWindow;
Ship *gShip;
Asteroid *gAsteroids[5];
Bullet *gBullet;
timer *gTimer;
static int asteroids = 6;

//////////////////////////////////////
// Ship control game variables
//////////////////////////////////////
float mfAngle;
int   miLeft, miRight;
int	  miLeftTime, miRightTime;
bool  mbThrust;
int   miThrustTime;
float mfXV, mfYV, mfV;
int   miLastTime;

/////////////////////////////////
// Asteroid control variables
/////////////////////////////////
float astAngle;
float astXV, astYV, astV;
float astSpeed; 

//////////////////////////////////
// Bullet control variables
//////////////////////////////////
float bulletXV, bulletYV, bulletV;
float bulletAngle;

////////////////////////////////////
//Main scene function prototypes
////////////////////////////////////
void DrawScene();
void DrawInfo();
void DrawInfoNumbers();
void DrawGameOver();
void IdleScene();
void Visible(int vis);
void ResizeWindow(GLsizei iWidth, GLsizei iHeight);
void KeyboardInput(unsigned char cKey, int iX, int iY);
void SpecialInput(int iKey, int iX, int iY);
void SpecialUp(int key, int x, int y);
void DebugInfo();

//////////////
//Main loop
//////////////
void main(int argc, char **argv) 
{ 
	FreeConsole();//use this to hide the console window 

	/////////////////////////////////
	//instatiate any objects here
	/////////////////////////////////
	gTimer = new timer();
	gTimer->stop();

	gShip = new Ship();
	gShip->mfPosX = 5.0f;
	gShip->mfPosY = 20.0f;

	gBullet = new Bullet();
	gBullet->mfPosX = gShip->mfPosX;
	gBullet->mfPosY = gShip->mfPosY;

	//the other 5...
	for(int i = 0; i < 6; ++i)
	{
		gAsteroids[i] = new Asteroid();
		gAsteroids[i]->mfPosX = rand()%50 - 1;
		gAsteroids[i]->mfPosY = rand()%50 - 1;
		gAsteroids[i]->miDirX = -1;
		gAsteroids[i]->miDirY = 1;
	}

	//////////////////////
	// Setup OpenGL here
	//////////////////////////
    GLsizei iWidth = 720.0; 
    GLsizei iHeight = 640.0; 

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_POINT_SMOOTH);

    glutInit(&argc, argv);
    glutInitWindowPosition( 0, 0);
    glutInitWindowSize(iWidth, iHeight);
    glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );
    glutCreateWindow( "Vlads Asteroids Game" );
    glutDisplayFunc(DrawScene);
	glutVisibilityFunc(Visible);
    glutIdleFunc(IdleScene);
    glutKeyboardFunc(KeyboardInput);
    glutSpecialFunc(SpecialInput);	
	glutSpecialUpFunc(SpecialUp);

    glutIgnoreKeyRepeat(1);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 50, 0, 50, 0, 50);
	glMatrixMode(GL_MODELVIEW); 
	glPointSize(3.0);
	glClearColor (0.1, 0.57, 0.8, 1.0);
	gameWindow = glutGetWindow();

    glutMainLoop(); 
}

void DebugInfo()
{
	//system("CLS");
	//cout << asteroids << endl;
}

///////////////////////////////////////
// Update all the game logic here
///////////////////////////////////////
void IdleScene()
{
	int time = glutGet(GLUT_ELAPSED_TIME);
	int delta  = time - miLastTime;

	if(gShip->mbAlive)
	{
		//gShip->UpdateShip(glutGet(GLUT_ELAPSED_TIME));

		if (miLeft) 
		{
			delta = time - miLeftTime;
			mfAngle = mfAngle + delta * 0.4f;
			miLeftTime = time;
		}
		if (miRight) 
		{
			delta = time - miRightTime;
			mfAngle = mfAngle - delta * 0.4f;
			miRightTime = time;
		}
		if (mbThrust) 
		{
			delta = time - miThrustTime;
			mfV = delta * 0.00004;
			mfXV = mfXV + cos(mfAngle * M_PI/180.0f) * mfV;
			mfYV = mfYV + sin(mfAngle  * M_PI/180.0f) * mfV;
			miThrustTime = time;
		}

		delta = time - miLastTime;

		//////////////////////////////////////
		//Update ship movement coords here
		//////////////////////////////////////
		gShip->mfPosX  = gShip->mfPosX  + mfXV * delta;
		gShip->mfPosY = gShip->mfPosY + mfYV * delta;
		gShip->mfPosX  = gShip->mfPosX  / 50.0;
		gShip->mfPosX  = (gShip->mfPosX  - floor(gShip->mfPosX )) * 50.0f;
		gShip->mfPosY = gShip->mfPosY / 50.0;
		gShip->mfPosY = (gShip->mfPosY - floor(gShip->mfPosY)) * 50.0f;

		///////////////////////////////////
		// Player bullet control
		///////////////////////////////////
		if(gTimer->elapsed_time() < gBullet->miExpire && gBullet->mbAlive)
		{
			//Advance bullet
			gBullet->mbShoot = true;
			gBullet->mbinUse = true;

			//firing calculations (fix me)
			float c = cos(mfAngle * M_PI/180.0);
			float s = sin(mfAngle * M_PI/180.0);

			gBullet->mfPosX = gBullet->mfPosX + 1.5 * c;
			gBullet->mfPosY = gBullet->mfPosY + 1.5 * s;
		}
		else
		{
			//reset bullet
			gTimer->stop();
			gBullet->mbShoot = false;
			gBullet->mbinUse = false;

			//Always spawn new bullet from the players current position
			gBullet->mfPosX = gShip->mfPosX;
			gBullet->mfPosY = gShip->mfPosY;
		}
	}

	///////////////////////////////////////////
	//Update asteroid movement coords here
	///////////////////////////////////////////

	//Set the intial velocity, angle, starting positions here
	astV = delta * 0.0004;
	astAngle = astAngle + delta * 0.05f;

	for(int i = 0; i < 6; ++i)
	{
		astXV = (gAsteroids[i]->mfSpeed * gAsteroids[i]->miDirX) + cos(astAngle * M_PI/180.0f) * astV ;
		astYV = (gAsteroids[i]->mfSpeed * gAsteroids[i]->miDirY) + sin(astAngle * M_PI/180.0f) * astV ;
		gAsteroids[i]->mfPosX = gAsteroids[i]->mfPosX  + astXV * astV ;
		gAsteroids[i]->mfPosY = gAsteroids[i]->mfPosY + astYV * astV ;
		gAsteroids[i]->mfPosX = gAsteroids[i]->mfPosX / 50.0f;
		gAsteroids[i]->mfPosX = (gAsteroids[i]->mfPosX - floor(gAsteroids[i]->mfPosX)) * 50.0f;
		gAsteroids[i]->mfPosY = gAsteroids[i]->mfPosY / 50.0f;
		gAsteroids[i]->mfPosY = (gAsteroids[i]->mfPosY - floor(gAsteroids[i]->mfPosY)) * 50.0f;
	}

	miLastTime = time;


	///////////////////////////////////////
	//Asteroid/Ship collision checking
	///////////////////////////////////////
	int tmpShipPosX = (int)gShip->mfPosX;
	int tmpShipPosY = (int)gShip->mfPosY;
	int tmpBulletPosX = (int)gBullet->mfPosX;
	int tmpBulletPosY = (int)gBullet->mfPosY;
	int shipRad = 1;
	int astRad = 1;
	int bulletRad = 1;

	//Asteroid/ship collision  
	if(gShip->mbAlive)
	{
		for(int  i = 0; i < 6; ++i)
		{
			if(gAsteroids[i]->mbAlive)
			{
				if(CollisionCheck((int)gAsteroids[i]->mfPosX, (int)gAsteroids[i]->mfPosY, astRad, tmpShipPosX, tmpShipPosY, shipRad))
				{
					gShip->mbAlive = false;
					gAsteroids[i]->mbAlive = false;
				}

				if(CollisionCheck((int)gAsteroids[i]->mfPosX, (int)gAsteroids[i]->mfPosY, astRad, tmpBulletPosX, tmpBulletPosY, bulletRad))
				{
					gAsteroids[i]->mbAlive = false;
					gBullet->mbAlive = false;
							
					//Decrement global asteroid count
					if(asteroids != 0)
					{
						--asteroids;
					}
				}
			}
		}
	}

	glutPostWindowRedisplay(gameWindow);

	//Debug
	DebugInfo();
}

void DrawScene()
{
	glClear(GL_COLOR_BUFFER_BIT); 

	glLoadIdentity();

	///////////////////////
	//Render scene here
	///////////////////////
	if(gShip->mbAlive  && asteroids != 0)
	{
		DrawInfo();
		DrawInfoNumbers();

		gShip->DrawShip(gShip->mfPosX, gShip->mfPosY, mfAngle, mbThrust);
		
		if(gBullet->mbShoot & gBullet->mbinUse)
		{
			//Draw bullet
			gBullet->Draw(gBullet->mfPosX, gBullet->mfPosY);
		}
	}
	else
	{
		DrawGameOver();
	}

	for(int i = 0; i < 6; ++i)
	{
		if(gAsteroids[i]->mbAlive)
		{
			gAsteroids[i]->Draw(gAsteroids[i]->mfPosX, gAsteroids[i]->mfPosY, astAngle); //changing the y-input variable to the current x-coordinate of the ship fixes the collision problem...
		}
	}

	glutSwapBuffers(); 
}

void Visible(int vis)
{
	if(vis == GLUT_VISIBLE) 
	{
		glutIdleFunc(IdleScene);
	}
}

void KeyboardInput(unsigned char cKey, int iX, int iY)
{
	switch(cKey)
	{
	case ' ':

		//bullet fire event
		if(!gBullet->mbinUse)
		{
			gTimer->restart(); //player cant fire another bullet unless 0.4 seconds have passed
			gBullet->mbAlive = true;
		}
		break;
	}

}

void SpecialInput(int iKey, int iX, int iY)
{
	switch (iKey)
	{
		case GLUT_KEY_UP:
			mbThrust = 1;
			miThrustTime = glutGet(GLUT_ELAPSED_TIME);
		break;

	  case GLUT_KEY_LEFT:
		  miLeft = 1;
		  miLeftTime = glutGet(GLUT_ELAPSED_TIME);
		break;

	  case GLUT_KEY_RIGHT:
		  miRight = 1;
		  miRightTime = glutGet(GLUT_ELAPSED_TIME);
		break;
	}
}

void SpecialUp(int iKey, int x, int y)
{
	switch (iKey) 
	{
		case GLUT_KEY_UP:
			mbThrust  = 0;
		break;

		case GLUT_KEY_LEFT:
			miLeft = 0;
		break;

		case GLUT_KEY_RIGHT:
			miRight = 0;
		break;
	}
}


void DrawInfo()
{
	glPushMatrix();
	glLoadIdentity();
	glColor3f(1.0, 1.0, 1.0);
	glRasterPos2i(1, 48);

	string s = "Asteroids remaining: ";

	void * font = GLUT_BITMAP_HELVETICA_18;
	for (string::iterator i = s.begin(); i != s.end(); ++i)
	{
		char c = *i;
		glutBitmapCharacter(font, c);
	}
	glPopMatrix();
}

//This function is used to update the games global asteroids info counter
void DrawInfoNumbers()
{
	static char numbers[7];
	char c;

	numbers[0] = '0';
	numbers[1] = '1';
	numbers[2] = '2';
	numbers[3] = '3';
	numbers[4] = '4';
	numbers[5] = '5';
	numbers[6] = '6';

	void * font = GLUT_BITMAP_HELVETICA_18;

	glPushMatrix();
	glLoadIdentity();
	glColor3f(1.0, 1.0, 1.0);
	glRasterPos2i(13, 48);

	switch(asteroids)
	{
	case 0:
			c = numbers[0];
			glutBitmapCharacter(font, c);
			break;

	case 1:
			c = numbers[1];
			glutBitmapCharacter(font, c);
			break;

	case 2:
			c = numbers[2];
			glutBitmapCharacter(font, c);
			break;

	case 3:
			c = numbers[3];
			glutBitmapCharacter(font, c);
			break;

		
	case 4:
			c = numbers[4];
			glutBitmapCharacter(font, c);
			break;

	case 5:
			c = numbers[5];
			glutBitmapCharacter(font, c);
			break;

	case 6:

			c = numbers[6];
			glutBitmapCharacter(font, c);
			break;
	}

	glPopMatrix();
}

void DrawGameOver()
{
	glPushMatrix();
	glLoadIdentity();
	glColor3f(1.0, 1.0, 1.0);
	glRasterPos2i(20, 25);

	string s = "GAME OVER";
	void * font = GLUT_BITMAP_HELVETICA_18;
	for (string::iterator i = s.begin(); i != s.end(); ++i)
	{
		char c = *i;
		glutBitmapCharacter(font, c);
	}
	glPopMatrix();
}